@@ -56,6 +56,9 @@
 #include "transactions/kgpgsignuid.h"
 #include "transactions/kgpgtransactionjob.h"
 
+#include <akonadi/contact/contacteditor.h>
+#include <akonadi/contact/contacteditordialog.h>
+#include <akonadi/contact/contactsearchjob.h>
 #include <KAction>
 #include <KActionCollection>
 #include <KDebug>
@@ -72,6 +75,7 @@
 #include <KMimeTypeTrader>
 #include <KProcess>
 #include <KRecentFilesAction>
+#include <KRun>
 #include <KSelectAction>
 #include <KService>
 #include <KShortcut>
@@ -103,8 +107,8 @@
 #include <QTextStream>
 #include <QWidget>
 #include <QtDBus/QtDBus>
-#include <kabc/addresseedialog.h>
-#include <kabc/stdaddressbook.h>
+#include <kabc/addresseelist.h>
+// #include <kabc/key.h> TODO
 #include <kio/global.h>
 #include <kjobtrackerinterface.h>
 #include <ktip.h>
@@ -176,6 +180,10 @@ KeysManager::KeysManager(QWidget *parent)
 	infoKey->setIcon(KIcon( QLatin1String( "document-properties-key" )));
 	infoKey->setText(i18n("K&ey Properties"));
 
+	QAction *openKeyUrl = actionCollection()->addAction(QLatin1String("key_url"), this, SLOT(slotOpenKeyUrl()));
+	openKeyUrl->setIcon(KIcon(QLatin1String("applications-internet")));
+	openKeyUrl->setText(i18n("&Open Key URL"));
+
 	editKey = actionCollection()->addAction(QLatin1String("key_edit"), this, SLOT(slotedit()));
 	editKey->setIcon(KIcon( QLatin1String( "utilities-terminal" )));
 	editKey->setText(i18n("Edit Key in &Terminal"));
@@ -195,6 +203,10 @@ KeysManager::KeysManager(QWidget *parent)
 	importKey->setText(i18n("&Import Key..."));
 	importKey->setShortcuts(KStandardShortcut::shortcut(KStandardShortcut::Paste));
 
+	m_sendEmail = actionCollection()->addAction(QLatin1String("send_mail"), this, SLOT(slotSendEmail()));
+	m_sendEmail->setIcon(KIcon(QLatin1String("mail-send")));
+	m_sendEmail->setText(i18n("Send Ema&il"));
+
 	QAction *newContact = actionCollection()->addAction(QLatin1String("add_kab"), this, SLOT(addToKAB()));
 	newContact->setIcon(KIcon( QLatin1String( "contact-new" )));
 	newContact->setText(i18n("&Create New Contact in Address Book"));
@@ -346,24 +358,27 @@ KeysManager::KeysManager(QWidget *parent)
 
 	m_popuppub = new KMenu(this);
 	m_popuppub->addAction(exportPublicKey);
-	m_popuppub->addAction(deleteKey);
+	m_popuppub->addAction(m_sendEmail);
+	m_popuppub->addAction(signMailUid);
 	m_popuppub->addAction(signKey);
 	m_popuppub->addAction(signUid);
-	m_popuppub->addAction(signMailUid);
+	m_popuppub->addAction(deleteKey);
 	m_popuppub->addAction(infoKey);
+	m_popuppub->addAction(openKeyUrl);
 	m_popuppub->addAction(editKey);
 	m_popuppub->addAction(refreshKey);
 	m_popuppub->addAction(createGroup);
-	m_popuppub->addAction(setDefaultKey);
 	m_popuppub->addSeparator();
 	m_popuppub->addAction(importAllSignKeys);
 
 	m_popupsec = new KMenu(this);
 	m_popupsec->addAction(exportPublicKey);
+	m_popupsec->addAction(m_sendEmail);
 	m_popupsec->addAction(signKey);
 	m_popupsec->addAction(signUid);
 	m_popupsec->addAction(signMailUid);
 	m_popupsec->addAction(infoKey);
+	m_popupsec->addAction(openKeyUrl);
 	m_popupsec->addAction(editKey);
 	m_popupsec->addAction(refreshKey);
 	m_popupsec->addAction(setDefaultKey);
@@ -395,8 +410,9 @@ KeysManager::KeysManager(QWidget *parent)
 	m_popupphoto->addAction(deletePhoto);
 
 	m_popupuid = new KMenu(this);
-	m_popupuid->addAction(signUid);
+	m_popupuid->addAction(m_sendEmail);
 	m_popupuid->addAction(signMailUid);
+	m_popupuid->addAction(signUid);
 	m_popupuid->addAction(delUid);
 	m_popupuid->addAction(setPrimUid);
 
@@ -423,7 +439,8 @@ KeysManager::KeysManager(QWidget *parent)
 	searchLayout->addWidget(m_listviewsearch);
 	searchLayout->addStretch();
 
-	KAction *searchLineAction = new KAction(i18nc("Name of the action that is a search line, shown for example in the toolbar configuration dialog", "Search Line"), this);
+	KAction *searchLineAction = new KAction(i18nc("Name of the action that is a search line, shown for example in the toolbar configuration dialog",
+			"Search Line"), this);
 	actionCollection()->addAction(QLatin1String( "search_line" ), searchLineAction);
 	searchLineAction->setDefaultWidget(searchWidget);
 
@@ -486,7 +503,8 @@ void KeysManager::slotGenerateKey()
 	if (kg->exec() == QDialog::Accepted) {
 		if (!kg->isExpertMode()) {
 			KGpgGenerateKey *genkey = new KGpgGenerateKey(this, kg->name(), kg->email(),
-					kg->comment(), kg->algo(), kg->size(), kg->days(), kg->expiration());
+					kg->comment(), kg->algo(), kg->size(), kg->days(), kg->expiration(),
+					kg->caps());
 
 			m_genkey = new KGpgTransactionJob(genkey);
 			connect(m_genkey, SIGNAL(result(KJob*)), SLOT(slotGenerateKeyDone(KJob*)));
@@ -499,7 +517,10 @@ void KeysManager::slotGenerateKey()
 
 			QString terminalApp(config.readPathEntry("TerminalApplication", QLatin1String( "konsole" )));
 			QStringList args;
-			args << QLatin1String( "-e" ) << KGpgSettings::gpgBinaryPath() << QLatin1String( "--gen-key" ) << QLatin1String( "--expert" );
+			args << QLatin1String( "-e" )
+					<< KGpgSettings::gpgBinaryPath()
+					<< QLatin1String("--gen-key")
+					<< QLatin1String("--expert");
 
 			QProcess *genKeyProc = new QProcess(this);
 			genKeyProc->start(terminalApp, args);
@@ -525,7 +546,6 @@ void KeysManager::slotOpenEditor()
 {
 	KgpgEditor *kgpgtxtedit = new KgpgEditor(this, imodel, Qt::Window);
 
-	connect(kgpgtxtedit, SIGNAL(encryptFiles(KUrl::List)), this, SIGNAL(encryptFiles(KUrl::List)));
 	connect(this, SIGNAL(fontChanged(QFont)), kgpgtxtedit, SLOT(slotSetFont(QFont)));
 
 	kgpgtxtedit->show();
@@ -632,7 +652,9 @@ void KeysManager::slotGenerateKeyDone(KJob *job)
 		break;
 	}
 	default:
-		KMessageBox::detailedError(this, i18n("gpg process did not finish. Cannot generate a new key pair."), genkey->gpgErrorMessage(), infomessage);
+		KMessageBox::detailedError(this,
+				i18n("gpg process did not finish. Cannot generate a new key pair."),
+				genkey->gpgErrorMessage(), infomessage);
 	}
 
 	m_genkey = NULL;
@@ -822,15 +844,21 @@ void KeysManager::slotregenerate()
 	KProcess *p1, *p2, *p3;
 
 	p1 = new KProcess(this);
-	*p1 << KGpgSettings::gpgBinaryPath() << QLatin1String( "--no-secmem-warning" ) << QLatin1String( "--export-secret-key" ) << regID;
+	*p1 << KGpgSettings::gpgBinaryPath()
+			<< QLatin1String("--no-secmem-warning")
+			<< QLatin1String("--export-secret-key")
+			<< regID;
 	p1->setOutputChannelMode(KProcess::OnlyStdoutChannel);
 
 	p2 = new KProcess(this);
-	*p2 << QLatin1String( "gpgsplit" ) << QLatin1String( "--no-split" ) << QLatin1String( "--secret-to-public" );
+	*p2 << QLatin1String("gpgsplit")
+			<< QLatin1String("--no-split")
+			<< QLatin1String("--secret-to-public");
 	p2->setOutputChannelMode(KProcess::OnlyStdoutChannel);
 
 	p3 = new KProcess(this);
-	*p3 << KGpgSettings::gpgBinaryPath() << QLatin1String( "--import" );
+	*p3 << KGpgSettings::gpgBinaryPath()
+			<< QLatin1String("--import");
 
 	p1->setStandardOutputProcess(p2);
 	p2->setStandardOutputProcess(p3);
@@ -853,7 +881,8 @@ void KeysManager::slotregenerate()
 void KeysManager::slotAddUid()
 {
 	if (m_adduid) {
-		KMessageBox::error(this, i18n("Another operation is still in progress.\nPlease wait a moment until this operation is complete."), i18n("Add New User Id"));
+		KMessageBox::error(this, i18n("Another operation is still in progress.\nPlease wait a moment until this operation is complete."),
+				i18n("Add New User Id"));
 		return;
 	}
 
@@ -872,7 +901,8 @@ void KeysManager::slotAddUid()
 	if (addUidWidget->exec() != QDialog::Accepted)
 		return;
 
-	m_adduid = new KGpgAddUid(this, iview->selectedNode()->getId(), keyUid->kLineEdit1->text(), keyUid->kLineEdit2->text(), keyUid->kLineEdit3->text());
+	m_adduid = new KGpgAddUid(this, iview->selectedNode()->getId(), keyUid->kLineEdit1->text(),
+			keyUid->kLineEdit2->text(), keyUid->kLineEdit3->text());
 	connect(m_adduid, SIGNAL(done(int)), SLOT(slotAddUidFin(int)));
 	m_adduid->start();
 }
@@ -972,32 +1002,52 @@ void KeysManager::slotSetPhotoSize(int size)
 
 void KeysManager::addToKAB()
 {
-	KABC::Key key;
 	KGpgNode *nd = iview->selectedNode();
 	if (nd == NULL)
 		return;
 
-	QString email(nd->getEmail());
+	Akonadi::ContactSearchJob * const job = new Akonadi::ContactSearchJob();
+	job->setLimit(1);
+	job->setQuery(Akonadi::ContactSearchJob::Email, nd->getEmail());
+	connect(job, SIGNAL(result(KJob*)), this, SLOT(slotAddressbookSearchResult(KJob*)));
 
-	KABC::AddressBook *ab = KABC::StdAddressBook::self();
-	if (!ab->load()) {
-		KMessageBox::sorry(this, i18n("Unable to contact the address book. Please check your installation."));
+	m_addIds[job] = nd;
+}
+
+void KeysManager::slotAddressbookSearchResult(KJob *job)
+{
+	KGpgNode * const nd = m_addIds.value(job, 0);
+
+	if (!nd)
 		return;
+
+	Akonadi::ContactSearchJob *searchJob = qobject_cast<Akonadi::ContactSearchJob*>(job);
+	Q_ASSERT(searchJob);
+	const KABC::Addressee::List addresseeList = searchJob->contacts();
+
+	m_addIds.take(job);
+
+	Akonadi::ContactEditorDialog *dlg;
+// 	KABC::Key key; TODO
+	if (!addresseeList.isEmpty()) {
+		dlg = new Akonadi::ContactEditorDialog(Akonadi::ContactEditorDialog::EditMode, this);
+		dlg->setContact(searchJob->items().first());
+	} else {
+		KABC::Addressee addressee;
+		addressee.setNameFromString(nd->getName());
+		addressee.setEmails(QStringList(nd->getEmail()));
+		dlg = new Akonadi::ContactEditorDialog(Akonadi::ContactEditorDialog::CreateMode, this);
+		dlg->editor()->setContactTemplate(addressee);
 	}
 
-	KABC::Addressee::List addresseeList(ab->findByEmail(email));
-	KToolInvocation::startServiceByDesktopName( QLatin1String( "kaddressbook" ));
-	QDBusInterface kaddressbook(QLatin1String( "org.kde.kaddressbook" ), QLatin1String( "/KAddressBook" ), QLatin1String( "org.kde.KAddressbook.Core" ));
-
-	if(!addresseeList.isEmpty())
-		kaddressbook.call( QLatin1String( "showContactEditor" ), addresseeList.first().uid());
-	else
-		kaddressbook.call( QLatin1String( "addEmail" ), QString(nd->getName() + QLatin1String( " <" ) + email + QLatin1Char( '>' )));
+	connect(dlg, SIGNAL(finished()), dlg, SLOT(deleteLater()));
+	dlg->show();
 }
 
 void KeysManager::slotManpage()
 {
-	KToolInvocation::startServiceByDesktopName( QLatin1String( "khelpcenter" ), QLatin1String("man:/gpg"), 0, 0, 0, "" , true);
+	KToolInvocation::startServiceByDesktopName(QLatin1String("khelpcenter"),
+			QLatin1String("man:/gpg"), 0, 0, 0, QByteArray(), true);
 }
 
 void KeysManager::slotTip()
@@ -1031,6 +1081,7 @@ void KeysManager::checkList()
 			m_revokeKey->setEnabled(exportList.at(0)->getType() == ITYPE_PAIR);
 			if (terminalkey)
 				editKey->setEnabled(false);
+			m_sendEmail->setEnabled(!exportList[0]->getEmail().isEmpty());
 			setDefaultKey->setEnabled(!imodel->isDefaultKey(exportList[0]));
 		}
 		break;
@@ -1512,7 +1563,8 @@ void KeysManager::slotShowPhoto()
 {
 	KService::List list(KMimeTypeTrader::self()->query(QLatin1String( "image/jpeg" )));
 	if (list.isEmpty()) {
-	KMessageBox::sorry(NULL, i18n("<qt>A viewer for JPEG images is not specified.<br/>Please check your installation.</qt>"), i18n("Show photo"));
+	KMessageBox::sorry(NULL, i18n("<qt>A viewer for JPEG images is not specified.<br/>Please check your installation.</qt>"),
+			i18n("Show photo"));
 	return;
 	}
 	KGpgNode *nd = iview->selectedNode();
@@ -1521,7 +1573,16 @@ void KeysManager::slotShowPhoto()
 	KService::Ptr ptr = list.first();
 
 	KProcess p;
-	p << KGpgSettings::gpgBinaryPath() << QLatin1String( "--no-tty" ) << QLatin1String( "--photo-viewer" ) << (ptr->desktopEntryName() + QLatin1String( " %i" )) << QLatin1String( "--edit-key" ) << parent->getId() << QLatin1String( "uid" ) << und->getId() << QLatin1String( "showphoto" ) << QLatin1String( "quit" );
+	p << KGpgSettings::gpgBinaryPath()
+			<< QLatin1String("--no-tty")
+			<< QLatin1String("--photo-viewer")
+			<< (ptr->desktopEntryName() + QLatin1String( " %i" ))
+			<< QLatin1String("--edit-key")
+			<< parent->getId()
+			<< QLatin1String("uid")
+			<< und->getId()
+			<< QLatin1String("showphoto")
+			<< QLatin1String("quit");
 	p.startDetached();
 }
 
@@ -1601,7 +1662,8 @@ void KeysManager::keyproperties()
 	case ITYPE_GSECRET:
 		if (KMessageBox::questionYesNo(this,
 			i18n("<p>This key is an orphaned secret key (secret key without public key.) It is currently not usable.</p>"
-				"<p>Would you like to regenerate the public key?</p>"), QString(), KGuiItem(i18n("Generate")), KGuiItem(i18n("Do Not Generate"))) == KMessageBox::Yes)
+					"<p>Would you like to regenerate the public key?</p>"),
+					QString(), KGuiItem(i18n("Generate")), KGuiItem(i18n("Do Not Generate"))) == KMessageBox::Yes)
 		slotregenerate();
 		return;
 	case ITYPE_PAIR:
@@ -1631,7 +1693,8 @@ void KeysManager::deleteGroup()
 	if (!nd || (nd->getType() != ITYPE_GROUP))
 		return;
 
-	int result = KMessageBox::warningContinueCancel(this, i18n("<qt>Are you sure you want to delete group <b>%1</b> ?</qt>",nd->getName()), QString(), KGuiItem(i18n("Delete"), QLatin1String( "edit-delete" )));
+	int result = KMessageBox::warningContinueCancel(this, i18n("<qt>Are you sure you want to delete group <b>%1</b> ?</qt>",
+			nd->getName()), QString(), KGuiItem(i18n("Delete"), QLatin1String("edit-delete")));
 	if (result != KMessageBox::Continue)
 		return;
 
@@ -1748,11 +1811,13 @@ void KeysManager::signkey()
 		if (nd->getEmail().isEmpty())
 			opt = i18n("<qt>You are about to sign key:<br /><br />%1<br />ID: %2<br />Fingerprint: <br /><b>%3</b>.<br /><br />"
 					"You should check the key fingerprint by phoning or meeting the key owner to be sure that someone "
-					"is not trying to intercept your communications.</qt>", nd->getName(), nd->getId().right(8), nd->getBeautifiedFingerprint());
+					"is not trying to intercept your communications.</qt>",
+					nd->getName(), nd->getId().right(8), nd->getBeautifiedFingerprint());
 		else
 			opt = i18n("<qt>You are about to sign key:<br /><br />%1 (%2)<br />ID: %3<br />Fingerprint: <br /><b>%4</b>.<br /><br />"
 					"You should check the key fingerprint by phoning or meeting the key owner to be sure that someone "
-					"is not trying to intercept your communications.</qt>", nd->getName(), nd->getEmail(), nd->getId().right(8), nd->getBeautifiedFingerprint());
+					"is not trying to intercept your communications.</qt>",
+					nd->getName(), nd->getEmail(), nd->getId().right(8), nd->getBeautifiedFingerprint());
 
 		if (KMessageBox::warningContinueCancel(this, opt) != KMessageBox::Continue) {
 			return;
@@ -1772,7 +1837,9 @@ void KeysManager::signkey()
 		}
 
 		if (KMessageBox::Continue != KMessageBox::warningContinueCancelList(this,
-				i18n("<qt>You are about to sign the following keys in one pass.<br/><b>If you have not carefully checked all fingerprints, the security of your communications may be compromised.</b></qt>"), signKeyList))
+				i18n("<qt>You are about to sign the following keys in one pass.<br/><b>If you have not carefully checked all fingerprints,"
+						" the security of your communications may be compromised.</b></qt>"),
+				signKeyList))
 			return;
 	}
 
@@ -1855,8 +1922,10 @@ void KeysManager::signuid()
 			signList.append(nd->toSignableNode());
 		}
 
-		if (KMessageBox::warningContinueCancelList(this, i18n("<qt>You are about to sign the following user ids in one pass.<br/><b>If you have not carefully checked all fingerprints, the security of your communications may be compromised.</b></qt>"),
-					signKeyList) != KMessageBox::Continue)
+		if (KMessageBox::warningContinueCancelList(this,
+				i18n("<qt>You are about to sign the following user ids in one pass.<br/><b>If you have not carefully checked all fingerprints,"
+						" the security of your communications may be compromised.</b></qt>"),
+				signKeyList) != KMessageBox::Continue)
 			return;
 	}
 
@@ -2001,9 +2070,13 @@ void KeysManager::signKeyOpenConsole(const QString &signer, const QString &keyid
 	KConfigGroup config(KGlobal::config(), "General");
 
 	KProcess process;
-	process << config.readPathEntry("TerminalApplication", QLatin1String( "konsole" ));
-	process << QLatin1String( "-e" ) << KGpgSettings::gpgBinaryPath() << QLatin1String( "--no-secmem-warning" ) << QLatin1String( "-u" ) << signer;
-	process << QLatin1String( "--default-cert-level" ) << QString(checking);
+	process << config.readPathEntry("TerminalApplication", QLatin1String("konsole"))
+			<< QLatin1String("-e")
+			<< KGpgSettings::gpgBinaryPath()
+			<< QLatin1String("--no-secmem-warning")
+			<< QLatin1String("-u") << signer
+			<< QLatin1String("--default-cert-level")
+			<< QString(checking);
 
 	if (!local)
 		process << QLatin1String( "--sign-key" ) << keyid;
@@ -2146,7 +2219,8 @@ void KeysManager::delsignkey()
 		return;
 	}
 
-	QString ask = i18n("<qt>Are you sure you want to delete signature<br /><b>%1</b><br />from user id <b>%2</b><br />of key: <b>%3</b>?</qt>", signMail, parentMail, parentKey);
+	QString ask = i18n("<qt>Are you sure you want to delete signature<br /><b>%1</b><br />from user id <b>%2</b><br />of key: <b>%3</b>?</qt>",
+			signMail, parentMail, parentKey);
 
 	if (KMessageBox::questionYesNo(this, ask, QString(), KStandardGuiItem::del(), KStandardGuiItem::cancel()) != KMessageBox::Yes)
 		return;
@@ -2171,6 +2245,23 @@ void KeysManager::delsignatureResult(int success)
 	}
 }
 
+void KeysManager::slotSendEmail()
+{
+	QStringList maillist;
+
+	foreach (const KGpgNode *nd, iview->selectedNodes()) {
+		if (nd->getEmail().isEmpty())
+			continue;
+
+		maillist << QLatin1Char('"') + nd->getName() + QLatin1String("\" <") + nd->getEmail() + QLatin1Char('>');
+	}
+
+	if (maillist.isEmpty())
+		return;
+
+	KToolInvocation::invokeMailer(maillist.join(QLatin1String(", ")), QString());
+}
+
 void KeysManager::slotedit()
 {
 	KGpgNode *nd = iview->selectedNode();
@@ -2185,8 +2276,13 @@ void KeysManager::slotedit()
 
 	KProcess *kp = new KProcess(this);
 	KConfigGroup config(KGlobal::config(), "General");
-	*kp << config.readPathEntry("TerminalApplication",QLatin1String( "konsole" ));
-	*kp << QLatin1String( "-e" ) << KGpgSettings::gpgBinaryPath() <<QLatin1String( "--no-secmem-warning" ) <<QLatin1String( "--edit-key" ) << nd->getId() << QLatin1String( "help" );
+	*kp << config.readPathEntry("TerminalApplication", QLatin1String("konsole"))
+			<< QLatin1String("-e")
+			<< KGpgSettings::gpgBinaryPath()
+			<< QLatin1String("--no-secmem-warning")
+			<< QLatin1String("--edit-key")
+			<< nd->getId()
+			<< QLatin1String("help");
 	terminalkey = nd->toKeyNode();
 	editKey->setEnabled(false);
 
@@ -2267,7 +2363,8 @@ void KeysManager::deleteseckey()
 
 	// delete a key
 	int result = KMessageBox::warningContinueCancel(this,
-			i18n("<p>Delete <b>secret</b> key pair <b>%1</b>?</p>Deleting this key pair means you will never be able to decrypt files encrypted with this key again.", nd->getNameComment()),
+			i18n("<p>Delete <b>secret</b> key pair <b>%1</b>?</p>Deleting this key pair means you will never be able to decrypt files encrypted with this key again.",
+					nd->getNameComment()),
 			QString(),
 			KGuiItem(i18n("Delete"), QLatin1String( "edit-delete" )));
 	if (result != KMessageBox::Continue)
@@ -2276,7 +2373,9 @@ void KeysManager::deleteseckey()
 	if (terminalkey == nd)
 		return;
 	if (m_delkey != NULL) {
-		KMessageBox::error(this, i18n("Another key delete operation is still in progress.\nPlease wait a moment until this operation is complete."), i18n("Delete key"));
+		KMessageBox::error(this,
+				i18n("Another key delete operation is still in progress.\nPlease wait a moment until this operation is complete."),
+				i18n("Delete key"));
 		return;
 	}
 
@@ -2303,7 +2402,9 @@ void KeysManager::secretKeyDeleted(int retcode)
 void KeysManager::confirmdeletekey()
 {
 	if (m_delkey) {
-		KMessageBox::error(this, i18n("Another key delete operation is still in progress.\nPlease wait a moment until this operation is complete."), i18n("Delete key"));
+		KMessageBox::error(this,
+				i18n("Another key delete operation is still in progress.\nPlease wait a moment until this operation is complete."),
+				i18n("Delete key"));
 		return;
 	}
 
@@ -2315,7 +2416,9 @@ void KeysManager::confirmdeletekey()
 
 	// do not delete a key currently edited in terminal
 	if ((!(pt & ~ITYPE_PAIR)) && (ndlist.at(0) == terminalkey) && (ndlist.count() == 1)) {
-		KMessageBox::error(this, i18n("Can not delete key <b>%1</b> while it is edited in terminal.", terminalkey->getBeautifiedFingerprint()), i18n("Delete key"));
+		KMessageBox::error(this,
+				i18n("Can not delete key <b>%1</b> while it is edited in terminal.",
+				terminalkey->getBeautifiedFingerprint()), i18n("Delete key"));
 		return;
 	} else if (pt == ITYPE_GROUP) {
 		deleteGroup();
@@ -2323,10 +2426,15 @@ void KeysManager::confirmdeletekey()
 	} else if (!(pt & ITYPE_GROUP) && (pt & ITYPE_SECRET) && (ndlist.count() == 1)) {
 		deleteseckey();
 		return;
+	} else if ((pt == ITYPE_UID) && (ndlist.count() == 1)) {
+		slotDelUid();
+		return;
 	}
 
 	if (pt & ~ITYPE_PAIR) {
-		KMessageBox::error(this, i18n("You have selected items that are not keys. They can not be deleted with this menu entry."), i18n("Delete key"));
+		KMessageBox::error(this,
+				i18n("You have selected items that are not keys. They can not be deleted with this menu entry."),
+				i18n("Delete key"));
 		return;
 	}
 
@@ -2349,7 +2457,9 @@ void KeysManager::confirmdeletekey()
 	}
 
 	if (secretKeyInside) {
-		int result = KMessageBox::warningContinueCancel(this, i18n("<qt>The following are secret key pairs:<br/><b>%1</b><br/>They will not be deleted.</qt>", secList.join( QLatin1String( "<br />" ))));
+		int result = KMessageBox::warningContinueCancel(this,
+				i18n("<qt>The following are secret key pairs:<br/><b>%1</b><br/>They will not be deleted.</qt>",
+						secList.join( QLatin1String( "<br />" ))));
 		if (result != KMessageBox::Continue)
 			return;
 	}
@@ -2357,7 +2467,11 @@ void KeysManager::confirmdeletekey()
 	if (keysToDelete.isEmpty())
 		return;
 
-	int result = KMessageBox::warningContinueCancelList(this, i18np("<qt><b>Delete the following public key?</b></qt>", "<qt><b>Delete the following %1 public keys?</b></qt>", keysToDelete.count()), keysToDelete, QString(), KStandardGuiItem::del());
+	int result = KMessageBox::warningContinueCancelList(this,
+			i18np("<qt><b>Delete the following public key?</b></qt>",
+					"<qt><b>Delete the following %1 public keys?</b></qt>",
+					keysToDelete.count()), keysToDelete, QString(),
+					KStandardGuiItem::del());
 	if (result != KMessageBox::Continue)
 		return;
 
@@ -2421,8 +2535,9 @@ void KeysManager::slotImport(const QString &text)
 
 	if (!KGpgImport::isKey(text) && KGpgDecrypt::isEncryptedText(text)) {
 		if (KMessageBox::questionYesNo(this,
-				i18n("<qt>The text in the clipboard does not look like a key, but like encrypted text.<br />Do you want to decrypt it first and then try importing it?</qt>"),
-					       i18n("Import from Clipboard")) != KMessageBox::Yes)
+				i18n("<qt>The text in the clipboard does not look like a key, but like encrypted text.<br />Do you want to decrypt it first"
+						" and then try importing it?</qt>"),
+						i18n("Import from Clipboard")) != KMessageBox::Yes)
 			return;
 
 		imp = new KGpgImport(this);
@@ -2455,7 +2570,7 @@ void KeysManager::slotImportDone(int result)
 
 	if (result != 0) {
 		KMessageBox::detailedSorry(this, i18n("Key importing failed. Please see the detailed log for more information."),
-				rawmsgs.join( QLatin1String( "\n")) , i18n("Key Import" ));
+				rawmsgs.join(QLatin1String("\n")) , i18n("Key Import"));
 	}
 
 	QStringList keys(import->getImportedIds(0x1f));
@@ -2466,7 +2581,7 @@ void KeysManager::slotImportDone(int result)
 		const QString msg(import->getImportMessage());
 		const QStringList keynames(import->getImportedKeys());
 
-		new KgpgDetailedInfo(this, msg, rawmsgs.join( QLatin1String( "\n") ), keynames, i18n("Key Import" ));
+		new KgpgDetailedInfo(this, msg, rawmsgs.join(QLatin1String("\n")), keynames, i18n("Key Import"));
 		if (needsRefresh)
 			imodel->refreshKeys(keys);
 		else
@@ -2634,6 +2749,46 @@ KeysManager::slotSetClip(int result)
 	m_trayicon->showMessage(QString(), i18n("Text successfully encrypted."), QLatin1String( "kgpg" ));
 }
 
+void
+KeysManager::slotOpenKeyUrl()
+{
+	KGpgNode *cur = iview->selectedNode();
+	if (cur == NULL)
+		return;
+
+	QString id;
+
+	switch (cur->getType()) {
+	case ITYPE_PAIR:
+	case ITYPE_PUBLIC: {
+		id = cur->toKeyNode()->getFingerprint();
+		break;
+		}
+	case ITYPE_GPAIR:
+	case ITYPE_GPUBLIC: {
+		id = cur->getId();
+		break;
+		}
+	default:
+		return;
+	}
+
+	const QStringList servers = KGpgSettings::infoServers();
+	if (servers.isEmpty())
+		return;
+
+	QString url = servers.first();
+
+	url.replace(QLatin1String("$$ID8$$"), id.right(8).toUpper());
+	url.replace(QLatin1String("$$ID16$$"), id.toUpper());
+	url.replace(QLatin1String("$$FPR$$"), id.toUpper());
+	url.replace(QLatin1String("$$id8$$"), id.right(8).toLower());
+	url.replace(QLatin1String("$$id16$$"), id.toLower());
+	url.replace(QLatin1String("$$fpr$$"), id.toLower());
+
+	new KRun(url, this);
+}
+
 void
 KeysManager::clipDecrypt()
 {
